package de.intevation.flys.client.client.ui;

import java.util.List;

import com.google.gwt.core.client.GWT;

import com.google.gwt.user.client.rpc.AsyncCallback;

import com.smartgwt.client.util.SC;

import com.smartgwt.client.widgets.Window;

import com.smartgwt.client.widgets.tree.Tree;
import com.smartgwt.client.widgets.tree.TreeGrid;

import com.smartgwt.client.widgets.layout.Layout;
import com.smartgwt.client.widgets.layout.VLayout;

import com.smartgwt.client.widgets.tree.events.LeafClickEvent;
import com.smartgwt.client.widgets.tree.events.LeafClickHandler;

import de.intevation.flys.client.shared.model.Artifact;
import de.intevation.flys.client.shared.model.ArtifactDescription;
import de.intevation.flys.client.shared.model.Data;
import de.intevation.flys.client.shared.model.DataList;
import de.intevation.flys.client.shared.model.DataItem;
import de.intevation.flys.client.shared.model.User;

import de.intevation.flys.client.client.FLYSConstants;
import de.intevation.flys.client.client.Config;

import de.intevation.flys.client.client.services.MetaDataService;
import de.intevation.flys.client.client.services.MetaDataServiceAsync;

import de.intevation.flys.client.shared.model.DataCageTree;
import de.intevation.flys.client.shared.model.DataCageNode;
import de.intevation.flys.client.shared.model.AttrList;

import de.intevation.flys.client.client.widgets.tree.AttributedTreeNode;

public class DatacageWindow
extends      Window
{
    protected MetaDataServiceAsync metaDataService =
        GWT.create(MetaDataService.class);

    protected FLYSConstants messages =
        GWT.create(FLYSConstants.class);


    protected Artifact artifact;
    protected User     user;

    protected TreeGrid treeGrid;

    protected Layout layout;


    public DatacageWindow(Artifact artifact, User user) {
        this.artifact = artifact;
        this.user     = user;

        setWidth(250);
        setHeight(400);

        layout = new VLayout();
        layout.setWidth100();

        addItem(layout);

        treeGrid = new TreeGrid();
        treeGrid.setLoadDataOnDemand(false);
        treeGrid.setWidth100();
        treeGrid.setHeight100();

        treeGrid.addLeafClickHandler(new LeafClickHandler() {
            @Override
            public void onLeafClicked(LeafClickEvent lce) {
                AttributedTreeNode atn = (AttributedTreeNode)lce.getLeaf();
                AttrList attrList = atn.getSpecialAttributes();
                if (attrList != null) {
                    for (int i = 0, N = attrList.size(); i < N; ++i) {
                        GWT.log("'" + attrList.getKey(i) + 
                            "' -> '" + attrList.getValue(i) + "'");
                    }
                }
            }
        });

        layout.addMember(treeGrid);

        String river =  findRiver();
        // TODO: i18n
        setTitle("Datenkorb: " + river);
        setShowMinimizeButton(false);
        setIsModal(true);
        setShowModalMask(true);
        setCanDragResize(true);

        centerInPage();

        triggerTreeBuilding();
    }

    protected void triggerTreeBuilding() {
        Config config = Config.getInstance();
        String url    = config.getServerUrl();
        String locale = config.getLocale();

        String artifactId = artifact.getUuid();
        String userId     = user != null ? user.identifier() : null;

        metaDataService.getMetaData(
            url, locale,
            artifactId,
            userId,
            new AsyncCallback<DataCageTree>() {
                public void onFailure(Throwable caught) {
                    GWT.log("Could not load meta data.");
                    SC.warn(caught.getMessage());
                }

                public void onSuccess(DataCageTree tree) {
                    GWT.log("Successfully loaded meta data.");
                    buildTree(tree);
                }
            });
    }

    protected void buildTree(DataCageTree tree) {
        treeGrid.setData(convertTree(tree));
    }

    protected static AttributedTreeNode convert(DataCageNode node) {
        List<DataCageNode> children = node.getChildren();

        AttributedTreeNode [] cs;

        if (children != null) {
            cs = new AttributedTreeNode[children.size()];

            for (int i = 0, N = children.size(); i < N; ++i) {
                cs[i] = convert(children.get(i));
            }
        }
        else {
            cs = new AttributedTreeNode[0];
        }

        return new AttributedTreeNode(
            node.getName(), node.getAttributes(), cs);
    }

    protected static Tree convertTree(DataCageTree tree) {
        DataCageNode root = tree.getRoot();

        List<DataCageNode> children = root.getChildren();

        AttributedTreeNode [] cs;

        if (children != null) {
            cs = new AttributedTreeNode[children.size()];

            for (int i = 0, N = children.size(); i < N; ++i) {
                cs[i] = convert(children.get(i));
            }
        }
        else {
            cs = new AttributedTreeNode[0];
        }

        AttributedTreeNode rn = new AttributedTreeNode(
            root.getName(), root.getAttributes(), cs);

        Tree r = new Tree();
        r.setRoot(rn);

        return r;
    }

    protected String findRiver() {

        ArtifactDescription adescr = artifact.getArtifactDescription();
        DataList [] data = adescr.getOldData();

        if (data != null && data.length > 0) {
            for (int i = 0; i < data.length; i++) {
                DataList dl = data[i];
                if (dl.getState().equals("state.winfo.river")) {
                    for (int j = dl.size()-1; j >= 0; --j) {
                        Data d = dl.get(j);
                        DataItem [] di = d.getItems();
                        if (di != null && di.length == 1) {
                           return d.getItems()[0].getStringValue();
                        }
                    }
                }
            }
        }

        return "";
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
